@@ -55,15 +55,8 @@ all standard actions can be accessed via the ``a`` prefix, like ``a.Text``.
 # imported when using from parce import *
 __all__ = (
     # important classes
-    'Cursor',
     'Document',
-
-    # often used names when defining languages
-    'default_action',
-    'default_target',
-    'Language',
-    'lexicon',
-    'skip',
+    'Cursor',
 
     # toplevel functions
     'events',
@@ -71,16 +64,57 @@ __all__ = (
     'root',
     'theme_by_name',
     'theme_from_file',
-    'tokens',
+
+    # often used names when defining languages
+    'Language',
+    'default_action',
+    'default_target',
+    'lexicon',
+    'skip',
 )
 
 from . import document, lexer, rule, ruleitem, treebuilder, work, util
-from .lexicon import lexicon
 from .language import Language
 from .document import Cursor
 from .pkginfo import version, version_string
 
 
+def lexicon(rules_func=None, **kwargs):
+    """Lexicon factory decorator.
+
+    Use this decorator to make a function in a Language class definition a
+    LexiconDescriptor object. The LexiconDescriptor is a descriptor, and when
+    calling it via the Language class attribute, a Lexicon is created, cached
+    and returned.
+
+    You can specify keyword arguments, that will be passed on to the Lexicon
+    object as soon as it is created.
+
+    The following keyword arguments are supported:
+
+    ``re_flags`` (0):
+        The flags that are passed to the regular expression compiler
+
+    ``consume`` (False):
+        When set to True, tokens originating from a rule that pushed this
+        lexicon are added to the target Context instead of the current.
+
+    The code body of the function should return (yield) the rules of the
+    lexicon, and is run with the Language class as first argument, as soon as
+    the lexicon is used for the first time.
+
+    You can also call the Lexicon object just as an ordinary classmethod, to
+    get the rules, e.g. for inclusion in a different lexicon.
+
+    """
+    from parce.lexicon import LexiconDescriptor
+    if rules_func and not kwargs:
+        return LexiconDescriptor(rules_func)
+    def lexicon(rules_func):
+        return LexiconDescriptor(rules_func, **kwargs)
+    return lexicon
+
+
 class Document(work.WorkerDocumentMixin, document.Document):
     """A Document that automatically keeps its contents tokenized.
 
@@ -233,11 +267,6 @@ def root(root_lexicon, text):
     return treebuilder.build_tree(root_lexicon, text)
 
 
-def tokens(root_lexicon, text):
-    """Convenience function that yields all the tokens from the text."""
-    return root(root_lexicon, text).tokens()
-
-
 def events(root_lexicon, text):
     """Convenience function that yields all the events from the text."""
     return lexer.Lexer([root_lexicon]).events(text)
@@ -260,7 +289,6 @@ default_action = util.Symbol("default_action")   #: denotes a default action for
 default_target = util.Symbol("default_target")   #: denotes a default target when no text matches
 
 
+#: A dynamic action that yields no tokens, thereby ignoring the matched text.
 skip = ruleitem.SkipAction()
-"""A dynamic action that yields no tokens, thereby ignoring the matched text."""
-
 
