@@ -27,7 +27,7 @@ import re
 
 from parce import Language, lexicon, default_action
 from parce.action import (
-    Comment, Delimiter, Escape, Name, Number, Operator, Pseudo, Text)
+    Character, Comment, Delimiter, Escape, Name, Number, Operator, Pseudo, Text)
 from parce.rule import arg, MATCH, bygroup, ifgroup, ifmember
 
 
@@ -43,9 +43,9 @@ class Latex(Language):
 
     @classmethod
     def common(cls):
-        yield r'(\\begin)(?:\s*(\{)(.*?)(\})|(?=[\W\d]))', \
-            bygroup(Name.Builtin, Delimiter, Name.Tag, Delimiter), \
-            cls.get_environment_target()
+        yield r'(\\begin)(?:\s*(?:(\{)(.*?)(\})|(\[))|(?=[\W\d]))', \
+            bygroup(Name.Builtin, Delimiter, Name.Tag, Delimiter, Delimiter.Bracket), \
+            ifgroup(5, cls.environment_option, cls.get_environment_target(MATCH[3]))
         yield r'(\\[^\W\d]+)(?:\s*(\[))?', bygroup(Name.Command, Delimiter.Bracket), \
             ifgroup(2, cls.option)
         yield r'\{\\(oe|OE|ae|AE|aa|AA|o|O|l|L|ss|SS)\}', Escape
@@ -63,21 +63,28 @@ class Latex(Language):
     def base(cls):
         """Basic stuff."""
         yield r'\\[#$&~^%{}_ ]', Escape
-        yield r'[&_^~]', Name.Command
+        yield r'[&_^~]', Character.Special
         yield r'\\\\', Delimiter.Terminator    # line termination TODO: better action?
         yield r'%', Comment, cls.comment
 
-    @lexicon
+    @lexicon(consume=True)
     def brace(cls):
         yield r'\}', Delimiter.Brace, -1
         yield from cls.root
 
     @lexicon
     def option(cls):
-        yield r'\]', Delimiter.Bracket, -1
+        yield r'(\])(?:\s*(\[))?', Delimiter.Bracket, ifgroup(2, 0, -1)
         yield from cls.common()
         yield default_action, Pseudo    # TODO: find better action
 
+    @lexicon
+    def environment_option(cls):
+        yield r'(\])\s*(?:(\{)(.*?)(\})|(\[))?', \
+            bygroup(Delimiter.Bracket, Delimiter, Name.Tag, Delimiter, Delimiter.Bracket), \
+                ifgroup(5, 0, (-1, ifgroup(4, cls.get_environment_target(MATCH[3]))))
+        yield from list(cls.option())[1:]   # not the first rule
+
     @lexicon
     def environment(cls):
         yield r'(\\end)(?:\s*(\{)(.*?)(\})|(?=[\W\d]))', \
@@ -91,7 +98,7 @@ class Latex(Language):
             bygroup(Name.Builtin, Delimiter, Name.Tag, Delimiter), -1
         yield from cls.math_common()
 
-    @lexicon
+    @lexicon(consume=True)
     def math(cls):
         yield arg(default=r'\}'), Delimiter, -1
         yield from cls.math_common()
@@ -109,12 +116,16 @@ class Latex(Language):
         yield default_action, Text.Math
 
     @classmethod
-    def get_environment_target(cls):
-        """Return environment target, can be overridden to support special environments."""
-        return ifmember(MATCH[3], MATH_ENVIRONMENTS, cls.environment_math, cls.environment)
+    def get_environment_target(cls, name):
+        """Return environment target for environment ``name``.
+
+        Can be overridden to support special environments.
+
+        """
+        return ifmember(name, MATH_ENVIRONMENTS, cls.environment_math, cls.environment)
 
     # ----------------------------- comments ---------------------------------
-    @lexicon(re_flags=re.MULTILINE)
+    @lexicon(re_flags=re.MULTILINE, consume=True)
     def comment(cls):
         yield '$', None, -1
         yield from cls.comment_common()
