@@ -296,7 +296,7 @@ class Node:
         """
         for parent, index in self.ancestors_with_index(upto):
             if index:
-                yield from util.tokens_bw(parent[index-1::-1])
+                yield from util.tokens(parent[:index], True)
 
     @property
     def query(self):
@@ -321,19 +321,6 @@ class Node:
 class Token(Node):
     """A Token instance represents a lexed piece of text.
 
-    A token has the following attributes:
-
-    `parent`:
-        the Context node to which the token was added
-    `pos`:
-        the position of the token in the original text
-    `end`:
-        the end position of the token in the original text
-    `text`:
-        the text of the token
-    `action`:
-        the action specified by the lexicon rule that created the token
-
     When a pattern rule in a lexicon matches the text, a Token is created. When
     that rule would create more than one Token from a single regular expression
     match, GroupToken objects are created instead, carrying the index of the
@@ -393,14 +380,20 @@ class Token(Node):
 
     __slots__ = "_parent", "pos", "text", "action"
 
-    is_token = True
-    group = None
+    is_token = True     #: Always True for Token
 
     def __init__(self, parent, pos, text, action):
-        self.parent = parent
-        self.pos = pos
-        self.text = text
-        self.action = action
+        self.parent = parent    #: The Context node to which the token was added
+        self.pos = pos          #: The position in the original text
+        self.text = text        #: The text of this token
+        self.action = action    #: The action specified by the lexicon rule that created the token
+
+    @property
+    def end(self):
+        """The end position of this token in the original text."""
+        return self.pos + len(self.text)
+
+    group = None        #: Always None for Token, an integer for :class:`GroupToken`
 
     def copy(self, parent=None):
         """Return a copy of the Token, but with the specified parent."""
@@ -452,10 +445,6 @@ class Token(Node):
     def __len__(self):
         return len(self.text)
 
-    @property
-    def end(self):
-        return self.pos + len(self.text)
-
     def forward_including(self, upto=None):
         """Yield all tokens in forward direction, including self."""
         yield self
@@ -468,10 +457,9 @@ class Token(Node):
 
     def forward_until_including(self, other):
         """Yield all tokens starting with us and upto and including the other."""
-        context, start_trail, end_trail = self.common_ancestor_with_trail(other)
-        if context:
-            for context, slice_ in context.slices(start_trail, end_trail):
-                yield from util.tokens(context[slice_])
+        r = self.range(other)
+        if r:
+            yield from r.tokens()
 
     def common_ancestor_with_trail(self, other):
         """Return a three-tuple(context, trail_self, trail_other).
@@ -501,6 +489,16 @@ class Token(Node):
                 return n, s_indices[s_i::-1], o_indices[::-1]
         return None, None, None
 
+    def range(self, other):
+        """Return a :class:`Range` from this token upto and including the other.
+
+        Returns None if the other :class:`Token` does not belong to the same
+        tree.
+
+        """
+        context, start_trail, end_trail = self.common_ancestor_with_trail(other)
+        if context:
+            return Range(context, start_trail, end_trail)
 
 
 class GroupToken(Token):
@@ -523,7 +521,7 @@ class GroupToken(Token):
     __slots__ = "group",
 
     def __init__(self, group, parent, pos, text, action):
-        self.group = group
+        self.group = group  #: The index of this token in a group (negated for the last token in a group)
         super().__init__(parent, pos, text, action)
 
     def copy(self, parent=None):
@@ -612,7 +610,7 @@ class Context(list, Node):
     """
     __slots__ = "lexicon", "_parent"
 
-    is_context = True
+    is_context = True   #: Always True for Context
 
     def __new__(cls, lexicon, parent):
         return list.__new__(cls)
@@ -722,47 +720,26 @@ class Context(list, Node):
                 else:
                     return height + 1
 
-    def tokens(self):
-        """Yield all Tokens, descending into nested Contexts."""
-        stack = []
-        i = 0
-        n = self
-        while True:
-            for i in range(i, len(n)):
-                m = n[i]
-                if m.is_token:
-                    yield m
-                else:
-                    stack.append(i)
-                    i = 0
-                    n = m
-                    break
-            else:
-                if stack:
-                    n = n.parent
-                    i = stack.pop() + 1
-                else:
-                    break
+    def tokens(self, reverse=False):
+        """Yield all Tokens, descending into nested Contexts.
 
-    def tokens_bw(self):
-        """Yield all Tokens, descending into nested Contexts, in backward direction."""
+        If ``reverse`` is set to True, yield all tokens in backward direction.
+
+        """
+        children = reversed if reverse else iter
         stack = []
-        n = self
-        i = len(n)
+        gen = children(self)
         while True:
-            for i in range(i - 1, -1, -1):
-                m = n[i]
-                if m.is_token:
-                    yield m
+            for n in gen:
+                if n.is_token:
+                    yield n
                 else:
-                    stack.append(i)
-                    n = m
-                    i = len(n)
+                    stack.append(gen)
+                    gen = children(n)
                     break
             else:
                 if stack:
-                    n = n.parent
-                    i = stack.pop()
+                    gen = stack.pop()
                 else:
                     break
 
@@ -949,52 +926,79 @@ class Context(list, Node):
             if node.is_token:
                 return node
 
-    def tokens_range(self, start=0, end=None):
-        """Yield all tokens (that completely fill this text range if specified).
+    def range(self, start=0, end=None):
+        """Return a :class:`Range`.
 
-        The first and last tokens may overlap with the start and end positions.
+        The ancestor of the range is the common ancestor of the tokens found at
+        start and end (or the context itself if start or end fall outside this
+        context). If start is 0 and end is None, the range encompasses the full
+        context.
+
+        Returns None if this context is empty.
 
         """
-        for context, slice_ in self.context_slices(start, end):
-            yield from util.tokens(context[slice_])
+        return Range.from_tree(self, start, end)
 
-    def context_slices(self, start=0, end=None):
-        """Yield (context, slice) tuples to yield tokens from.
 
-        Yield the tokens using the context[slice] notation. The first and
-        last tokens that would be yielded from the iterables may overlap with
-        the start and end positions.
+class Range:
+    """A Range denotes a range of a tree structure.
+
+    A range is defined by an ancestor context and possibly empty lists pointing
+    to the start and end token, if specified. If both trails are not specified,
+    the range encompasses the full context.
+
+    """
+    def __init__(self, ancestor, start_trail=None, end_trail=None):
+        self.ancestor = ancestor                #: The specified ancestor
+        self.start_trail = start_trail or []    #: The specified start trail (empty list by default)
+        self.end_trail = end_trail or []        #: The specified end trail (empty list by default)
+
+    def __repr__(self):
+        return "<{} {} [{}:{}]>".format(type(self).__name__, self.ancestor.lexicon, self.pos, self.end)
+
+    @property
+    def pos(self):
+        """The position of the first token in our range."""
+        n = self.ancestor
+        for i in self.start_trail:
+            n = n[i]
+        return n.pos
+
+    @property
+    def end(self):
+        """The end position of the last token in our range."""
+        n = self.ancestor
+        for i in self.end_trail:
+            n = n[i]
+        return n.end
+
+    @classmethod
+    def from_tree(cls, tree, start=0, end=None):
+        """Create a Range.
+
+        The ancestor is the common ancestor of the tokens found at start and
+        end (or the tree itself if start or end fall outside the range of the
+        tree). If start is 0 and end is None, the range encompasses the full
+        tree.
+
+        Returns None if the tree is empty.
 
         """
-        context, start_trail, end_trail = self.context_trails(start, end)
-        if context:
-            yield from context.slices(start_trail, end_trail)
-
-    def context_trails(self, start=0, end=None):
-        """Return a three-tuple(context, start_trail, end_trail).
-
-        This can be used to denote a range of the tree structure in slices. The
-        returned context is the common ancestor of the tokens found at start
-        and end (or the current node if start or end fall outside the range of
-        the node). The trails are (possibly empty) lists of indices pointing to
-        the start and end token, if any.
-
-        """
-        if not self:
-            return None, None, None  # empty
-        context = self
-        if end is not None and end < self.end:
+        if not tree:
+            return # empty
+        context = tree
+        if end is not None and end < tree.end:
             if end <= start:
-                return None, None, None
-            end_trail = self.find_token_left_with_trail(end)[1]
+                return
+            end_trail = tree.find_token_left_with_trail(end)[1]
             if not end_trail:
-                return None, None, None
+                return
         else:
             end_trail = []
         if start > 0:
-            start_trail = self.find_token_with_trail(start)[1]
+            start_trail = tree.find_token_with_trail(start)[1]
             if not start_trail:
-                return None, None, None
+                return
             if end_trail:
                 # find the youngest common ancestor
                 for n, (i, j) in enumerate(zip(start_trail, end_trail)):
@@ -1006,25 +1010,25 @@ class Context(list, Node):
                     del end_trail[:n]
         else:
             start_trail = []
-        return context, start_trail, end_trail
+        return cls(context, start_trail, end_trail)
 
-    def slices(self, start_trail, end_trail, target_factory=None):
-        """Yield from the current context (context, slice) tuples.
+    def slices(self, target_factory=None):
+        """Yield (context, slice) tuples.
 
-        ``start_trail`` and ``end_trail`` both are lists of indices that
-        point to descendant tokens of this context. The yielded slices
-        include these tokens.
+        The yielded slices include the tokens at the end of start and end
+        trail.
 
-        If you specify a ``target_factory``, it should be a TargetFactory
-        object, and it will be updated along with the yielded slices.
+        If you specify a ``target_factory``, it should be a
+        :class:`~.target.TargetFactory` object, and it will be updated along
+        with the yielded slices.
 
         """
-        if start_trail:
-            start = start_trail[0]
-            if len(start_trail) > 1:
+        if self.start_trail:
+            start = self.start_trail[0]
+            if len(self.start_trail) > 1:
                 ancestors = []
-                n = self[start]
-                for i in start_trail[1:]:
+                n = self.ancestor[start]
+                for i in self.start_trail[1:]:
                     ancestors.append((n, i))
                     n = n[i]
                 yield ancestors[-1][0], slice(i, None) # include start token
@@ -1035,21 +1039,31 @@ class Context(list, Node):
                 start += 1
         else:
             start = 0
-        if end_trail:
-            end = end_trail[0]
-            if len(end_trail) == 1:
-                yield self, slice(start, end + 1)    # include end token
+        if self.end_trail:
+            end = self.end_trail[0]
+            if len(self.end_trail) == 1:
+                yield self.ancestor, slice(start, end + 1)    # include end token
             else:
-                yield self, slice(start, end)
-                n = self[end]
-                for end in end_trail[1:-1]:
+                yield self.ancestor, slice(start, end)
+                n = self.ancestor[end]
+                for end in self.end_trail[1:-1]:
                     target_factory and target_factory.push(n.lexicon)
                     yield n, slice(end)
                     n = n[end]
                 target_factory and target_factory.push(n.lexicon)
-                yield n, slice(end_trail[-1] + 1)   # include end token
+                yield n, slice(self.end_trail[-1] + 1)   # include end token
         else:
-            yield self, slice(start, None)
+            yield self.ancestor, slice(start, None)
+
+    def tokens(self):
+        """Yield all tokens in this range.
+
+        The first and last tokens may overlap with the start and end positions.
+
+        """
+        for context, slice_ in self.slices():
+            yield from util.tokens(context[slice_])
+
 
 
 def make_tokens(lexemes, parent=None):
