@@ -40,20 +40,17 @@ The TreeBuilder is designed so that it is possible to perform tokenizing in a
 background thread, and even interrupt tokenizing when changes are to be applied
 while processing previous changes.
 
-The :class:`BackgroundTreeBuilder` provides an implementation using Python
-threads.
-
 """
 
-import operator
 import threading
 
+from parce import util
 from parce.lexer import Lexer
-from parce.util import Observable, tokens
 from parce.target import TargetFactory
 from parce.tree import Context, make_tokens
 from parce.treebuilderutil import (
-    BuildResult, ReplaceResult, Changes, get_prepared_lexer, new_tree)
+    BuildResult, ReplaceResult, Changes, ancestors_with_index,
+    get_prepared_lexer, new_tree)
 
 
 def build_tree(root_lexicon, text, pos=0):
@@ -73,7 +70,7 @@ def build_tree(root_lexicon, text, pos=0):
     return root
 
 
-class TreeBuilder(Observable):
+class TreeBuilder(util.Observable):
     """Build a tree from parsing the text.
 
     The root node of the tree is in the ``root`` instance attribute.
@@ -163,6 +160,7 @@ class TreeBuilder(Observable):
 
     def __init__(self, root_lexicon=None):
         super().__init__()
+        self._lock = threading.Lock()
         self.root = Context(root_lexicon, None)
         self.busy = False
         self.changes = []
@@ -170,7 +168,7 @@ class TreeBuilder(Observable):
     def tree(self, text):
         """Convenience method to build a tree and return the root node."""
         self.rebuild(text)
-        return self.get_root(True)
+        return self.root
 
     def rebuild(self, text, root_lexicon=False, start=0, removed=0, added=None):
         """Tokenize the modified part of the text again and update the tree.
@@ -201,45 +199,34 @@ class TreeBuilder(Observable):
         Calls :meth:`build_new_tree` and :meth:`replace_tree` to do the actual work.
 
         """
-        if added is None:
-            added = len(text) - start
-        self.lock(True)
-        self.changes.append((text, root_lexicon, start, removed, added))
-        self.lock(False)
+        self.add_changes(text, root_lexicon, start, removed, added)
         if not self.busy:
             self.busy = True
             self.start_processing()
 
+    def add_changes(self, text, root_lexicon, start, removed, added):
+        """Add the changes to our changes list, but do not rebuild immediately.
+
+        The arguments are the same as for :meth:`rebuild`.
+
+        """
+        if added is None:
+            added = len(text) - start
+        with self._lock:
+            self.changes.append((text, root_lexicon, start, removed, added))
+
     def build_new_tree(self, text, root_lexicon, start, removed, added):
         """Build a new tree without yet modifying the current tree.
 
+        This method is called by :meth:`process`. Returns a ``BuildResult``
+        five-tuple with ``tree``, ``start``, ``end``, ``offset`` and
+        ``lexicons`` values. The ``start`` and ``end`` are the insert positions
+        in the old tree.
+
         Tokens from the current tree are reused as much as possible. From
         tokens at the tail (after the end of the modified region) the pos
         attribute is updated if necessary.
 
-        The arguments:
-
-        ``text``
-            The text to parse. Always the entire text, also when only a small
-            portion was changed.
-
-        ``root_lexicon``
-            The root lexicon to use. False means no change; can be None or any
-            Lexicon. If not False, the tree is always rebuilt completely.
-
-        ``start``
-            Position of the change.
-
-        ``removed``
-            The number of removed characters.
-
-        ``added``
-            The number of added characters.
-
-        Returns a ``BuildResult`` five-tuple with ``tree``, ``start``, ``end``,
-        ``offset`` and ``lexicons`` values. The ``start`` and ``end`` are the
-        insert positions in the old tree.
-
         The new ``tree`` is intended to replace a part of, or the whole old
         tree. If ``start`` == 0 and ``lexicons`` is not None; the whole tree
         can be replaced. (In this case; the root lexicon might have changed!)
@@ -288,12 +275,12 @@ class TreeBuilder(Observable):
         while True:
             # when restarting, see if we can reuse (part of) the new tree
             if tree:
-                result = get_prepared_lexer(tree, text, start)
+                result = get_prepared_lexer(tree, text, start, True)
                 if result:
                     lexer, events, tokens = result
                     t = tokens[0]
                     context = t.parent
-                    for p, i in t.ancestors_with_index():
+                    for p, i in ancestors_with_index(t):
                         del p[i+1:]
                     del context[-1]
                 else:
@@ -367,19 +354,18 @@ class TreeBuilder(Observable):
                                     else:
                                         tail = False
                         break # break the for loop to restart at new start pos
-                if peek:
-                    if tokens[-1].pos > peek:
-                        # call peek with a copy of the current tree.
-                        copied_tree = tree.copy()
-                        # remove the first empty context
-                        t = copied_tree[0]
-                        while t and t.is_context:
-                            t = t[0]
-                        while t.is_context and not t:
-                            t = t.parent
-                            del t[0]
-                        self.peek(lowest_start, copied_tree)
-                        peek = 0
+                if peek and tokens[-1].pos > peek:
+                    # call peek with a copy of the current tree.
+                    copied_tree = tree.copy()
+                    # remove the first empty context
+                    t = copied_tree[0]
+                    while t and t.is_context:
+                        t = t[0]
+                    while t.is_context and not t:
+                        t = t.parent
+                        del t[0]
+                    self.peek(lowest_start, copied_tree)
+                    peek = 0
             else:
                 # we ran till the end, also return the open lexicons
                 return BuildResult(tree, lowest_start, len(text), 0, lexer.lexicons[1:])
@@ -419,7 +405,7 @@ class TreeBuilder(Observable):
             # find the context that really changes, adjust trails
             i = 0
             for i, (s, e) in enumerate(zip(start_trail, end_trail)):
-                if s == e and tree.is_context and len(tree) == 1:
+                if s == e and len(tree) == 1 and tree[0].is_context:
                     context = context[s]
                     tree = tree[0]
                 else:
@@ -445,7 +431,7 @@ class TreeBuilder(Observable):
                         for n in s:
                             n.parent = t
                         if offset:
-                            for n in tokens(s):
+                            for n in util.tokens(s):
                                 n.pos += offset
                     if t:
                         t = t[l]    # t can be empty if len(end_trail) == 1, is last iteration anyway
@@ -512,7 +498,7 @@ class TreeBuilder(Observable):
         You can reimplement this method to notify others of the change.
 
         """
-        for t in tokens(context[index:]):
+        for t in util.tokens(context[index:]):
             t.pos += offset
 
     def invalidate_context(self, context):
@@ -529,42 +515,11 @@ class TreeBuilder(Observable):
         """
         self.emit("invalidate", context)
 
-    def get_root(self, wait=False, callback=None, args=(), kwargs={}):
-        """Return the root element of the completed tree.
-
-        This is simply the ``root`` instance attribute, but this method only
-        returns the tree when the ``busy`` attribute is False.
-
-        If wait is True, this call blocks until tokenizing is done, and the
-        full tree is returned. If wait is False, None is returned if the tree
-        is still busy being built.
-
-        If a callback is given and tokenizing is still busy, that callback is
-        called once when tokenizing is ready. If given, args and kwargs are the
-        arguments the callback is called with, defaulting to () and {},
-        respectively.
-
-        Note that, for the lifetime of a TreeBuilder, the root element is always
-        the same. The root element is also accessible in the `root` attribute.
-        But using this method you can be sure that you are dealing with a
-        complete and fully intact tree.
-
-        """
-        if self.busy:
-            if callback:
-                if args or kwargs:
-                    callback = lambda: callback(*args, **kwargs)
-                self.connect("finished", callback, True)
-            if not wait:
-                return
-            self.wait()
-        return self.root
-
     def get_changes(self):
         """Get and combine the stored change requests in a Changes object.
 
         This may only be called from the same thread that also performs the
-        :meth:`rebuild()`.
+        :meth:`rebuild`.
 
         """
         c = Changes()
@@ -604,10 +559,10 @@ class TreeBuilder(Observable):
         self.process_started()
         start = end = -1
         lexicons = False    # no change
-        self.lock(True)
+        self._lock.acquire()
         c = self.get_changes()
         while c and c.has_changes():
-            self.lock(False)
+            self._lock.release()
             yield "build"
             result = self.build_new_tree(c.text, c.root_lexicon, c.start, c.removed, c.added)
             yield "replace"
@@ -617,9 +572,8 @@ class TreeBuilder(Observable):
             end = r.end if end == -1 else max(c.new_position(end), r.end)
             if r.lexicons is not None:
                 lexicons = r.lexicons
-            self.lock(True)
+            self._lock.acquire()
             c = self.get_changes()
-        yield "finish"
         if start != -1:
             self.start = start
         if end != -1:
@@ -627,17 +581,9 @@ class TreeBuilder(Observable):
         if lexicons is not False:
             self.lexicons = lexicons
         self.busy = False
-        self.lock(False)
-        self.process_finished()
+        self._lock.release()
         yield "done"
-
-    def wait(self):
-        """Implement to wait for completion if a background job is running.
-
-        The default implementation does nothing, and immediately returns.
-
-        """
-        pass
+        self.process_finished()
 
     def peek(self, start, tree):
         """This is called from :meth:`build_new_tree` with a sneak preview tree.
@@ -667,15 +613,6 @@ class TreeBuilder(Observable):
         """
         self.emit("peek", start, tree)
 
-    def lock(self, acquire):
-        """Acquire lock (True) or release lock (False). Does nothing by default.
-
-        If you want to run the full update and replace jobs in a background
-        thread, you may need locking, to prevent changes from going unnoticed.
-
-        """
-        pass
-
     def process_started(self):
         """Called at the start ot the tree building process.
 
@@ -697,42 +634,3 @@ class TreeBuilder(Observable):
         self.emit("finished")
 
 
-class BackgroundTreeBuilder(TreeBuilder):
-    """A TreeBuilder that can tokenize a text in a Python thread.
-
-    In BackgroundTreeBuilder, :meth:`rebuild()` returns immediately, because
-    :meth:`start_processing()` has been reimplemented to call itself in a
-    background thread.
-
-    You can continue adding changes while previous changes are processed;
-    the tree builder will immediately adapt to the new changes.
-
-    To be sure you get a completed tree, call ``get_root(True)``.
-
-    """
-    def __init__(self, root_lexicon=None):
-        super().__init__(root_lexicon)
-        self.job = None
-        self._lock = threading.Lock()
-
-    def lock(self, acquire):
-        """Reimplemented to actually lock/unlock."""
-        self._lock.acquire() if acquire else self._lock.release()
-
-    def start_processing(self):
-        """Reimplemented to call start_processing in a background thread."""
-        self.job = threading.Thread(target=super().start_processing)
-        self.job.start()
-
-    def wait(self):
-        """Reimplemented to await our background thread if active."""
-        job = self.job
-        if job:
-            job.join()
-
-    def process_finished(self):
-        """Reimplemented to clear the job attribute."""
-        self.job = None
-        super().process_finished()
-
-
