@@ -18,12 +18,6 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 """
-This package contains small modules with the name corresponding
-to the language definition in the parce.lang module.
-
-Every module defines one function, ``examples()``, that yields
-tuples (root_lexicon, text).
-
-Those can be used to show short examples of text and the tree that
-results from parsing that text with that root lexicon.
+This directory contains example and test files for the bundled language
+definitions.
 """
